#include "display.h"
#include "pins.h"
#include <avr/io.h>    // For pin manipulation
#include <util/delay.h> // For delay functions

// Constructor to initialize the CD4511
Display::Display(bool inverse)
{
    // Set data pins as output
    DDRD |= (1 << DP_DATA_A) | (1 << DP_DATA_B) | (1 << DP_DATA_C) | (1 << DP_DATA_D);

    // Set latch pins as output
    DDRC |= (1 << DP_LATCH_1) | (1 << DP_LATCH_2);
    DDRB |= (1 << DP_LATCH_3) | (1 << DP_LATCH_4);

    // Set the inverse flag
    _inverse = inverse;
}

// Write a digit (0-9) to the CD4511
void Display::write(uint8_t digit)
{
    uint8_t pattern = _digitToPattern(digit);
    
    // Shift the pattern to align with PD3–PD6
    pattern <<= 3;

    // Write the pattern to PORTD, preserving unrelated bits
    PORTD = (PORTD & 0x87) | (pattern & 0x78); // Preserve bits 7, 2–0; update bits 6–3
}

// Convert the digit (0-9) to the 4-bit pattern for the CD4511
uint8_t Display::_digitToPattern(uint8_t digit)
{
    uint8_t patterns[10] = {
        0b0000, // 0
        0b0001, // 1
        0b0010, // 2
        0b0011, // 3
        0b0100, // 4
        0b0101, // 5
        0b0110, // 6
        0b0111, // 7
        0b1000, // 8
        0b1001  // 9
    };

    // If inverse logic is set, flip the bits
    if (_inverse) {
        return ~patterns[digit] & 0x0F;  // Mask to keep only the lower 4 bits
    } else {
        return patterns[digit];
    }
}

// Latch the display (make only selected display active)
void Display::_latch(uint8_t displayNum)
{
    // Turn off all latches first (to ensure only one display is active at a time)
    PORTC &= ~((1 << DP_LATCH_1) | (1 << DP_LATCH_2));
    PORTB &= ~((1 << DP_LATCH_3) | (1 << DP_LATCH_4));

    // Activate the appropriate display latch
    switch (displayNum) {
        case 1:
            PORTC |= (1 << DP_LATCH_1);
            break;
        case 2:
            PORTC |= (1 << DP_LATCH_2);
            break;
        case 3:
            PORTB |= (1 << DP_LATCH_3);
            break;
        case 4:
            PORTB |= (1 << DP_LATCH_4);
            break;
        default:
            // Invalid display number, do nothing
            return;     
    }
}

// Activate display at selected position and display given digit
void Display::update(uint8_t displayNum, uint8_t digit)
{
    // Latch selected display
    _latch(displayNum);

    // Write digit to the display
    write(digit);

    // Allow some time for the displays to update before continuing
    _delay_ms(1);
}